#!/bin/sh

domain="$1"
user="$2"
group="$3"

usage() {
  echo Takes three args: 1\) domain, 2\) user and 3\) group. Fourth and optional is platform.
}

if [ $# -lt 3 ] ; then
  usage
  exit 1
fi

if ! id $user &> /dev/null ; then 
  echo this user \($user\) doesn\'t exist, please try again ; 
  exit 2
fi

if test $(groups $user | cut -d ':' -f 2 | grep $group | wc -l) = 0 ; then
  echo Sorry, the user specified \($user\) is not in the group specified \($group\), so this
  echo script won\'t work. Please fix and try again.
  exit 4
fi

if [ $# -lt 4 ] ; then
  full_path=$(aegircd "$domain")
  if [ $? == 1 -o $? == 2 ] ; then
    echo could not find one and only one platform for $domain , specify it as a third argument...
    echo eg: run find /var/aegir/platforms/ -iname \"$domain\" for yourself
  fi

  # pretty ugly but works
  platform=$(dirname $(dirname "$full_path"))
else
  platform="$4"
fi

source="$platform/sites/$domain"
target="/home/$user/$domain"

if [[ ! -d $source ]] ; then
  # can't access source
  echo Can\'t access source \($source\) directory, probably you wrote the domain wrong
  exit 3
fi

# we create the directory for the user
if [[ ! -d $target ]] ; then
  echo That user doesn\'t have a homedir, creating $target
  mkdir /srv/chroot/$user && ln -s /srv/chroot/$user /home/$user
  mkdir -p "$target"
  chown "$user" "$target"
  chmod g+s "$target"
fi

# we move the data from the site in the home dir, and make links to the drupal tree
for part in modules files themes libraries; do
  if [ -e $target/$part ] ; then
    echo target $target/$part already exists
  else
    mv $source/$part $target/$part && ln -s $target/$part $source/$part
    chown -R "$user" "$target/$part"
    setfacl -R -m group:$group:rwx "$target/$part"
    setfacl -R -m default:group:$group:rwx "$target/$part"
  fi
done
